package String::LibTRGM::Spell;

use strict;
$String::LibTRGM::Spell::VERSION = 0.02;

sub new {
  my ($proto, %opts) = @_;
  my $class = ref($proto) || $proto;
  my $self = {};
  $self->{_OUT} = $opts{out} && ref($opts{out}) eq 'CODE' ? $opts{out} : sub { print $_[0] };
  $self->{_TOLERANCE} = $opts{tolerance} || 0;
  $self->{_TRIG} = [];
  $self->{_LENGTH} = $opts{length};
  bless $self, $class;
  $self->get_bestmatch($opts{bestmatch}) if $opts{bestmatch};
  return $self;
}

sub set_length {
  my ($self, $len) = @_;
  my $old = $self->{_LENGTH};
  $self->{_LENGTH} = $len;
  return $old;
}

sub set_sink {
  my ($self, $code) = @_;
  return undef unless $code && ref($code) eq 'CODE';
  my $old = $self->{_OUT};
  $self->{_OUT} = $code;
  return $old;
}

sub set_tolerance {
  my ($self, $tolerance) = @_;
  my $old = $self->{_TOLERANCE};
  $self->{_TOLERANCE} = $tolerance + 0;
  return $old;
}

sub get_bestmatch {
  my ($self, $string) = @_;
  return $self->{_TRIG} = [] unless(length $string);
  my @res = ();
  my $state = 0;
  my ($val, $str) = ('','');
  my $mnv;
  my %hashstr;
  while ($string =~ /(.)/gs){
    my $char = $1;
    last if $self->{_LENGTH} && @res >= $self->{_LENGTH};
    $state == 0 && $char =~ /\S/  and $val .= $char, $state = 1, next;
    $state == 1 && $char =~ /\s/  and $state = 2, next;
    $state == 1                   and $val .= $char, next;
    $state == 2 && !defined($mnv) and $mnv = $val*$self->{_TOLERANCE};
    $state == 2 && $val < $mnv    and last;
    $state == 2 && $char eq '"'   and $state = 3, next;
    $state == 3 && $char eq '\\'  and $state = 4, next;
    $state == 3 && $char eq '"'   and $state = 5;
    $state == 3                   and $str .= $char, next;
    $state == 4                   and $str .= $char, $state = 3, next;
    $state == 5 && $hashstr{$str} and $val=$str='', $state=6;
    $state == 5                   and push(@res, [$val, $str]), $hashstr{$str}=1, $val=$str='', $state=6;
    $state == 6 && $char eq '|'   and $state=0;
  }
  return $self->{_TRIG} = \@res;
}

sub get_strings {
  my $self = shift;
  return wantarray ? @{$self->{_TRIG}} : scalar(@{$self->{_TRIG}});
}

sub get_select_widget {
  my ($self, %opts) = @_;
  return unless @{$self->{_TRIG}};
  my $name = $opts{name};
  my $multiple = $opts{multiple} ? ' multiple' : '';
  my $ex_attr  = $opts{ex_attr} || '';
  $ex_attr = " $ex_attr" if length $ex_attr;
  &{$self->{_OUT}}(qq[<select name="$name"$multiple$ex_attr>\n]);
  for my $trig (@{$self->{_TRIG}}){
    my $val = $self->html_escape($trig->[1]);
    &{$self->{_OUT}}(qq[<option value="$val">$val\n]);
  }
  &{$self->{_OUT}}(qq[</select>]);
  return '';
}

sub get_checkboxes {
  my ($self, %opts) = @_;
  return unless @{$self->{_TRIG}};
  my $name = $opts{name};
  my $beg = $opts{beg} || sub { '' };
  my $end = $opts{end} || sub { '<br>' };
  for my $trig (@{$self->{_TRIG}}){
    my $val = $self->html_escape($trig->[1]);
    &{$self->{_OUT}}(qq[<input type="checkbox" name="$name" value="$val">].&$beg($val).$val.&$end($val).qq[\n]);
  }
  return '';
}

sub get_list {
  my ($self, %opts) = @_;
  return unless @{$self->{_TRIG}};
  my $list = $opts{ordered} ? 'ol':'ul';
  my $beg = $opts{beg} || sub { '' };
  my $end = $opts{end} || sub { '' };
  &{$self->{_OUT}}(qq[<$list>\n]);
  for my $trig (@{$self->{_TRIG}}){
    my $val = $self->html_escape($trig->[1]);
    &{$self->{_OUT}}(qq[<li>].&$beg($val).$val.&$end($val).qq[\n]);
  }
  &{$self->{_OUT}}(qq[</$list>]);
  return '';
}

# Borrowed from HTML::Mason::Tools
sub html_escape {
  my ($self, $text) = @_;
  my %html_escape = ('&' => '&amp;', '>'=>'&gt;', '<'=>'&lt;', '"'=>'&quot;');
  my $html_escape = join('', keys %html_escape);
  $text =~ s/([$html_escape])/$html_escape{$1}/mgoe;
  return $text;
}


1;


=head1 NAME

String::LibTRGM::Spell - A collection of utilities to work with  output from
C<String::LibTRGM::bestmatch> function.

=head1 SYNOPSIS

  use String::LibTRGM::Spell;
  my $output = '';
  my $obj = String::LibTRGM::Spell->new(out => sub { $output.=$_[0] });
  $obj->get_bestmatch(String::LibTRGM::bestmatch(...));
  $obj->get_select_widget(name => 'trigrm');
  print $output;

=head1 DESCRIPTION

=over 4

=item new

Creates new C<String::LibTRGM::Spell> object. Arguments:

=over 8

=item out

Sets the subroutine to call for output. Default: C<sub { print $_[0] }>.

=item tolerance

Sets the tolerance (string, those similarity is less then C<tolerance*maximal string similarity> will be discarded).
Default value is C<0>.

=item bestmatch

Setting this parameter is equivalent to calling C<get_bestmatch> just after C<new>.

=item length

Sets the number of items to parse. Default is C<undef> that means that all the string will be parsed.

=back

=item get_bestmatch

Parses the string returned by C<String::LibTRGM::bestmatch> function and stores the result for later use.
The only parameter is the string to parse. Calling without parameter or with empty string as a parameter
clears the previous stored value.

=item set_length

Sets the number of items to parse by the next C<get_bestmatch> call. Return the old value.

=item get_strings

In scalar context return the number of strings, parsed by C<get_bestmatch>. In array context
returns list of array refs like C<[similarity, string]>, ordered by similarity descending.

=item set_tolerance

Sets the tolerance value and returns the old one (see C<new>).

=item set_sink

Sets the subroutine that will handle all output. Retusns the previous sink.

=item get_select_widget

Returns the string representation of HTML C<select> widget. Arguments:

=over 8

=item name

The value of the C<name> attribute of C<select> tag.

=item multiple

If true the C<multiple> attribute will be added to C<select> tag.

=item ex_attr

The string containing another attributes that will be added to C<select> tag.

=back

=item get_checkboxes

Returns the string representation of a list of HTML checkboxes. Arguments:

=over 8 

=item name

The value of the C<name> attribute for C<input> tag.

=item beg

A subroutine that those output will be inserted before description of checkbox. This subroutine should expect
the value of checkbox as it's only parameter. Default value is C<sub { '' }> i.e. nothing will precede the value.

=item end

A subroutine that those output will be inserted after description of checkbox. This subroutine should expect
the value of checkbox as it's only parameter. Default value is C<sub { 'E<lt>brE<gt>' }>.

=back

=item get_list

Returns the string representation of a HTML list. Arguments:

=over 8

=item ordered

If set to true the list will be the ordered one (C<ol>), otherwise unordered (C<ul>). Default is unordered.

=item beg

A subroutine that those output will be inserted before each item in list. This subroutine should expect
the value of item as it's only parameter. Default value is C<sub { '' }>.

=item end

A subroutine that those output will be inserted after each item in list. This subroutine should expect
the value of item as it's only parameter. Default value is C<sub { '' }>.

=back
  
=back

=cut
